/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.dao;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.security.IndustrialSector;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.persistence.PersistenceUtil;

/**
 * 
 * @author clemens.duepmeier
 */
public class UserDao {
	private static final Logger logger = LoggerFactory.getLogger(UserDao.class);

	public List<User> getUsers() {
		EntityManager em = PersistenceUtil.getEntityManager();

		List<User> users = (List<User>) em.createQuery("select user from User user order by user.userName")
				.getResultList();

		return users;
	}

	public User getUser(String userName) {
		EntityManager em = PersistenceUtil.getEntityManager();

		User user = null;
		try {
			user = (User) em.createQuery("select user from User user where user.userName=:userName")
					.setParameter("userName", userName).getSingleResult();
		} catch (NoResultException e) {
			// we do nothing here; just return null
		}
		return user;
	}

	public User getUser(long userId) {
		EntityManager em = PersistenceUtil.getEntityManager();

		User user = (User) em.find(User.class, Long.valueOf(userId));

		return user;
	}

	public List<IndustrialSector> getIndustrialSectors() {
		EntityManager em = PersistenceUtil.getEntityManager();

		List<IndustrialSector> sectors = new ArrayList<IndustrialSector>();

		sectors = em.createQuery("select s from IndustrialSector s order by s.sector").getResultList();

		return sectors;
	}

	public void persist(User user) throws Exception {
		EntityManager em = PersistenceUtil.getEntityManager();

		try {
			em.getTransaction().begin();
			em.persist(user);
			em.getTransaction().commit();
		} catch (Exception e) {
			em.getTransaction().rollback();
			throw e;
		}
	}

	public boolean checkAndPersist(User user, PersistType pType, PrintWriter out) {
		EntityManager em = PersistenceUtil.getEntityManager();

		User existingUser = this.getUser(user.getUserName());
		if (existingUser != null) {
			if (pType == pType.ONLYNEW) {
				if (out != null) {
					out.println("Warning: User with this name already exists; choose another user name");
				}
				return false;
			}
		}
		try {
			em.getTransaction().begin();
			if (existingUser != null && (pType == pType.MERGE)) {
				if (out != null) {
					out.println("Notice: user with this user name already exist; will update with this entry");
				}
				// delete first the existing one, we will use the new one
				em.remove(existingUser);
			}

			em.persist(user);
			em.getTransaction().commit();
			return true;
		} catch (Exception e) {
			logger.error("Cannot persist user {}", user.getUserName());
			logger.error("stack trace is:", e);
			em.getTransaction().rollback();
		}
		return false;
	}

	public void remove(User user) {
		EntityManager em = PersistenceUtil.getEntityManager();
		try {
			em.getTransaction().begin();
			User managed = em.find(User.class, user.getId());
			em.remove(managed);
			em.getTransaction().commit();

		} catch (Exception e) {
		}
	}

	public void merge(User user) throws MergeException {
		EntityManager em = PersistenceUtil.getEntityManager();
		try {
			em.getTransaction().begin();
			em.merge(user);
			em.getTransaction().commit();

		} catch (Exception e) {
			logger.error("Cannot persist changes to user {}", user.getUserName());
			logger.error("stack trace is:", e);
			em.getTransaction().rollback();
			throw new MergeException("Cannot merge changes to user  " + user.getUserName() + " into the database");
		}
	}
}
