/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.model.process;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import de.fzk.iai.ilcd.service.model.IProcessVO;
import de.fzk.iai.ilcd.service.model.common.IMultiLangString;
import de.fzk.iai.ilcd.service.model.enums.CompletenessValue;
import de.fzk.iai.ilcd.service.model.enums.TypeOfProcessValue;
import de.fzk.iai.ilcd.service.model.enums.TypeOfQuantitativeReferenceValue;
import de.fzk.iai.ilcd.service.model.process.IComplianceSystem;
import de.fzk.iai.ilcd.service.model.process.IQuantitativeReference;
import de.fzk.iai.ilcd.service.model.process.IReferenceFlow;
import de.fzk.iai.ilcd.service.model.process.IReview;
import de.iai.ilcd.model.common.Classification;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.DataSetVersion;
import de.iai.ilcd.model.common.GlobalReference;
import de.iai.ilcd.model.common.MultiLanguageString;
import de.iai.ilcd.model.common.MultiLanguageText;
import de.iai.ilcd.model.common.Uuid;
import de.iai.ilcd.model.common.exception.FormatException;
import de.iai.ilcd.model.datastock.DataStock;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
@Table(uniqueConstraints = @UniqueConstraint(columnNames = { "UUID", "MAJORVERSION", "MINORVERSION", "SUBMINORVERSION" }))
public class Process extends DataSet implements Serializable, IProcessVO {

	@Embedded
	protected ProcessName processName = new ProcessName();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "synonyms"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "process_synonyms", joinColumns = @JoinColumn(name = "process_id", referencedColumnName = "ID")))
	protected MultiLanguageString synonyms = new MultiLanguageString();

	@Enumerated(EnumType.STRING)
	protected TypeOfProcessValue type;

	protected boolean parameterized;

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "useAdvice"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "process_useAdvice", joinColumns = @JoinColumn(name = "process_id", referencedColumnName = "ID")))
	MultiLanguageText useAdvice;

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "technicalPurpose"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "process_purpose", joinColumns = @JoinColumn(name = "process_id", referencedColumnName = "ID")))
	MultiLanguageText technicalPurpose;

	@Embedded
	protected TimeInformation timeInformation;

	@Embedded
	protected Geography geography;

	protected String format;

	@ManyToOne(cascade = CascadeType.ALL)
	protected GlobalReference ownerReference;

	@OneToOne(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	protected LCIMethodInformation lCIMethodInformation;

	@Enumerated(EnumType.STRING)
	protected CompletenessValue completeness;

	@OneToMany(cascade = CascadeType.ALL, targetEntity = Review.class)
	protected List<IReview> reviews = new ArrayList<IReview>();

	@Embedded
	protected AccessInformation accessInformation;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY, targetEntity = ComplianceSystem.class)
	protected Set<IComplianceSystem> complianceSystems = new HashSet<IComplianceSystem>();

	@ManyToOne(cascade = CascadeType.ALL)
	protected GlobalReference approvedBy;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@OrderBy("internalId")
	protected List<Exchange> exchanges = new LinkedList<Exchange>();

	@OneToOne(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	protected InternalQuantitativeReference internalReference;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	protected List<LciaResult> lciaResults = new LinkedList<LciaResult>();

	protected boolean resultsIncluded;

	protected boolean exchangesIncluded;

	@ManyToMany(mappedBy = "processes", cascade = CascadeType.ALL)
	protected Set<DataStock> containingDataStocks = new HashSet<DataStock>();

	@Transient
	boolean accessRestricted = false;

	public Process() {
	}

	// TODO this is VERY ugly
	public Process(IProcessVO voProcess) {
		this.setUuid(new Uuid(voProcess.getUuidAsString()));
		this.setPermanentUri(voProcess.getPermanentUri());
		this.setHref(voProcess.getHref());
		ProcessName pname = new ProcessName();
		String[] nameParts = voProcess.getDefaultName().split(";");
		if (nameParts.length > 0) {
			pname.setBasePart(new MultiLanguageString(nameParts[0]));
		}
		if (nameParts.length > 1) {
			pname.setRoutePart(new MultiLanguageString(nameParts[1]));
		}
		if (nameParts.length > 2) {
			pname.setLocationPart(new MultiLanguageString(nameParts[2]));
		}
		if (nameParts.length > 3) {
			pname.setUnitPart(new MultiLanguageString(nameParts[3]));
		}
		this.setProcessName(pname);
		if (voProcess.getSynonyms() != null) {
			this.setSynonyms(new MultiLanguageString(voProcess.getSynonyms()));
		}
		if (voProcess.getClassification() != null) {
			this.setClassification(new Classification(voProcess.getClassification()));
		}
		if (voProcess.getType() != null) {
			this.setType(voProcess.getType());
		}
		this.setParameterized(voProcess.getParameterized());
		if (voProcess.getDescription() != null) {
			this.setDescription(new MultiLanguageText(voProcess.getDescription()));
		}
		if (voProcess.getUseAdvice() != null) {
			this.setUseAdvice(new MultiLanguageText(voProcess.getUseAdvice()));
		}
		if (voProcess.getLocation() != null) {
			Geography poGeography = new Geography();
			poGeography.setLocation(voProcess.getLocation());
			this.setGeography(poGeography);
		}
		TimeInformation poTimeInformation = new TimeInformation();
		if (voProcess.getTimeInformation() != null && voProcess.getTimeInformation().getReferenceYear() != null) {
			poTimeInformation.setReferenceYear(voProcess.getTimeInformation().getReferenceYear());
		}
		if (voProcess.getTimeInformation() != null && voProcess.getTimeInformation().getValidUntil() != null) {
			poTimeInformation.setValidUntil(voProcess.getTimeInformation().getValidUntil());
		}
		this.setTimeInformation(poTimeInformation);
		if (voProcess.getDataSetVersion() != null) {
			DataSetVersion dataSetVersion = new DataSetVersion();
			try {
				dataSetVersion = DataSetVersion.parse(voProcess.getDataSetVersion());
			} catch (FormatException ex) {
				// ignore; this should not happen
			}
			this.setVersion(dataSetVersion);
		}
//		this.setFormat("ILCD format");
	}

	@Override
	public AccessInformation getAccessInformation() {
		return accessInformation;
	}

	public void setAccessInformation(AccessInformation accessInformation) {
		this.accessInformation = accessInformation;
	}

	public CompletenessValue getCompleteness() {
		return completeness;
	}

	@Override
	public CompletenessValue getCompletenessProductModel() {
		return completeness;
	}

	public void setCompleteness(CompletenessValue completeness) {
		this.completeness = completeness;
	}

	public List<Exchange> getExchanges() {
		return exchanges;
	}

	public List<Exchange> getExchanges(String direction) {
		List<Exchange> filtered = new LinkedList<Exchange>();

		for (Exchange exchange : exchanges) {
			if (exchange.getExchangeDirection().toString().equals(direction)) {
				filtered.add(exchange);
			}
		}

		return filtered;
	}

	protected void setExchanges(List<Exchange> exchanges) {
		this.exchanges = exchanges;
	}

	public void addExchange(Exchange exchange) {
		exchanges.add(exchange);
	}

	@Override
	public Set<IComplianceSystem> getComplianceSystems() {
		return this.complianceSystems;
	}

	/**
	 * Convenience method for returning compliance systems as List in order to user p:dataList (primefaces)
	 * 
	 * @return List of compliance systems
	 */
	public List<IComplianceSystem> getComplianceSystemsAsList() {
		return new ArrayList<IComplianceSystem>(this.getComplianceSystems());
	}

	protected void setComplianceSystems(Set<IComplianceSystem> compliances) {
		this.complianceSystems = compliances;
	}

	public void addComplianceSystem(ComplianceSystem compliance) {
		complianceSystems.add(compliance);
	}

	@Override
	public GlobalReference getApprovedBy() {
		return approvedBy;
	}

	public void setApprovedBy(GlobalReference approvedByReference) {
		this.approvedBy = approvedByReference;
	}

	@Override
	public String getFormat() {
		return format;
	}

	public void setFormat(String format) {
		this.format = format;
	}

	public Geography getGeography() {
		return geography;
	}

	public void setGeography(Geography geography) {
		this.geography = geography;
	}

	@Override
	public String getLocation() {
		if (geography != null)
			return geography.getLocation();
		else
			return null;
	}

	@Override
	public Boolean getParameterized() {
		return parameterized;
	}

	public void setParameterized(boolean parameterized) {
		this.parameterized = parameterized;
	}

	public boolean isResultsIncluded() {
		return resultsIncluded;
	}

	@Override
	public Boolean getHasResults() {
		return this.resultsIncluded;
	}

	public void setResultsIncluded(boolean resultsIncluded) {
		this.resultsIncluded = resultsIncluded;
	}

	public boolean isExchangesIncluded() {
		return exchangesIncluded;
	}

	public void setExchangesIncluded(boolean exchangesIncluded) {
		this.exchangesIncluded = exchangesIncluded;
	}

	@Override
	public LCIMethodInformation getLCIMethodInformation() {
		return lCIMethodInformation;
	}

	public void setLCIMethodInformation(LCIMethodInformation lciMethodInformation) {
		this.lCIMethodInformation = lciMethodInformation;
	}

	public List<LciaResult> getLciaResults() {
		return lciaResults;
	}

	protected void setLciaResults(List<LciaResult> lciaResults) {
		this.lciaResults = lciaResults;
	}

	public void addLciaResult(LciaResult result) {
		lciaResults.add(result);
	}

	@Override
	public GlobalReference getOwnerReference() {
		return ownerReference;
	}

	public void setOwnerReference(GlobalReference ownerReference) {
		this.ownerReference = ownerReference;
	}

	public ProcessName getProcessName() {
		return processName;
	}

	@Override
	public MultiLanguageString getName() {
		MultiLanguageString fullName = processName.getFullName();

		return fullName;
	}

	@Override
	public MultiLanguageString getBaseName() {
		return processName.getBasePart();
	}

	public void setProcessName(ProcessName processName) {
		this.processName = processName;
	}

	public InternalQuantitativeReference getInternalReference() {
		return internalReference;
	}

	public void setInternalReference(InternalQuantitativeReference internalReference) {
		this.internalReference = internalReference;
	}

	@Override
	public IQuantitativeReference getQuantitativeReference() {
		return new QuantitativeReference(internalReference, exchanges);
	}

	public List<Exchange> getReferenceExchanges() {
		List<Exchange> refExchanges = new ArrayList<Exchange>();

		if (internalReference == null || internalReference.referenceIds == null) {
			return refExchanges;
		}
		if (internalReference.referenceIds.size() <= 0) {
			return refExchanges;
		}
		for (Integer intObject : internalReference.referenceIds) {
			for (Exchange exchange : exchanges) {
				// because we cannot guarantie that internId's runnin from 0 and are continously allocated, we have to
				// search for them
				if (exchange.getInternalId() == intObject.intValue()) {
					refExchanges.add(exchange);
					break;
				}
			}
		}
		return refExchanges;
	}

	public MultiLanguageString getOtherReference() {
		if (internalReference == null) {
			return null;
		}
		return internalReference.getOtherReference();
	}

	@Override
	public List<IReview> getReviews() {
		return reviews;
	}

	protected void setReviews(List<IReview> reviews) {
		this.reviews = reviews;
	}

	public void addReview(Review review) {
		if (!reviews.contains(review)) {
			reviews.add(review);
		}
	}

	@Override
	public MultiLanguageString getSynonyms() {
		return synonyms;
	}

	public void setSynonyms(MultiLanguageString synonyms) {
		this.synonyms = synonyms;
	}

	@Override
	public TimeInformation getTimeInformation() {
		return timeInformation;
	}

	public void setTimeInformation(TimeInformation timeInformation) {
		this.timeInformation = timeInformation;
	}

	@Override
	public TypeOfProcessValue getType() {
		return type;
	}

	public void setType(TypeOfProcessValue type) {
		this.type = type;
	}

	@Override
	public MultiLanguageText getUseAdvice() {
		return useAdvice;
	}

	public void setUseAdvice(MultiLanguageText useAdvice) {
		this.useAdvice = useAdvice;
	}

	@Override
	public MultiLanguageText getTechnicalPurpose() {
		return technicalPurpose;
	}

	public void setTechnicalPurpose(MultiLanguageText technicalPurpose) {
		this.technicalPurpose = technicalPurpose;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.process.Process[id=" + id + "]";
	}

	@Override
	public String getOverallQuality() {
		String overallQuality = null;

		return overallQuality;
	}

	@Override
	public boolean isAccessRestricted() {
		return accessRestricted;
	}

	@Override
	public void setAccessRestricted(boolean restricted) {
		accessRestricted = restricted;
	}

	public Set<DataStock> getContainingDataStocks() {
		return containingDataStocks;
	}

	private void setContainingDataStocks(Set<DataStock> containingDataStocks) {
		this.containingDataStocks = containingDataStocks;
	}

	public void addToDataStock(DataStock stock) {
		if (!containingDataStocks.contains(stock)) {
			containingDataStocks.add(stock);
		}
	}

	public void removeFromStock(DataStock stock) {
		if (containingDataStocks.contains(stock)) {
			stock.deleteProcess(this);
			containingDataStocks.remove(stock);
		}
	}

	public void removeFromAllStocks() {
		List<DataStock> all = new ArrayList<DataStock>();
		all.addAll(containingDataStocks);

		for (DataStock stock : all)
			this.removeFromStock(stock);
	}

}

class QuantitativeReference implements Serializable, IQuantitativeReference {

	List<Exchange> exchanges;

	InternalQuantitativeReference internalReference;

	public QuantitativeReference(InternalQuantitativeReference internalReference, List<Exchange> exchanges) {
		this.exchanges = exchanges;
		this.internalReference = internalReference;
	}

	@Override
	public List<IReferenceFlow> getReferenceFlows() {
		List<IReferenceFlow> refExchanges = new ArrayList<IReferenceFlow>();

		if (internalReference == null || internalReference.referenceIds == null) {
			return refExchanges;
		}
		if (internalReference.referenceIds.size() <= 0) {
			return refExchanges;
		}
		for (Integer intObject : internalReference.referenceIds) {
			for (Exchange exchange : exchanges) {
				// because we cannot guarantie that internId's runnin from 0 and are continously allocated, we have to
				// search for them
				if (exchange.getInternalId() == intObject.intValue()) {
					refExchanges.add(exchange);
					break;
				}
			}
		}
		return refExchanges;
	}

	@Override
	public IMultiLangString getFunctionalUnit() {
		if (internalReference != null)
			return internalReference.getOtherReference();
		else
			return null;
	}

	@Override
	public TypeOfQuantitativeReferenceValue getType() {
		if (internalReference != null)
			return internalReference.getType();
		else
			return null;
	}
}
