/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.process;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;

import de.fzk.iai.ilcd.service.model.common.IGlobalReference;
import de.fzk.iai.ilcd.service.model.enums.TypeOfReviewValue;
import de.fzk.iai.ilcd.service.model.process.IDataQualityIndicator;
import de.fzk.iai.ilcd.service.model.process.IReview;
import de.fzk.iai.ilcd.service.model.process.IScope;
import de.iai.ilcd.model.common.GlobalReference;
import de.iai.ilcd.model.common.MultiLanguageText;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
public class Review implements Serializable, IReview {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@Enumerated(EnumType.STRING)
	protected TypeOfReviewValue type;

	@OneToMany(cascade = CascadeType.ALL, targetEntity = ScopeOfReview.class)
	protected Set<IScope> scopes = new HashSet<IScope>();

	@OneToMany(cascade = CascadeType.ALL)
	protected Set<DataQualityIndicator> qualityIndicators = new HashSet<DataQualityIndicator>();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "reviewDetails"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "review_reviewDetails", joinColumns = @JoinColumn(name = "review_id", referencedColumnName = "ID")))
	MultiLanguageText reviewDetails;

	@OneToMany(cascade = CascadeType.ALL)
	Set<GlobalReference> referencesToReviewers = new HashSet<GlobalReference>();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "otherDetails"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "review_otherDetails", joinColumns = @JoinColumn(name = "review_id", referencedColumnName = "ID")))
	MultiLanguageText otherReviewDetails = new MultiLanguageText();

	public Review() {

	}

	public Review(TypeOfReviewValue type) {
		this.type = type;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public MultiLanguageText getOtherReviewDetails() {
		return otherReviewDetails;
	}

	public void setOtherReviewDetails(MultiLanguageText otherReviewDetails) {
		this.otherReviewDetails = otherReviewDetails;
	}

	public Set<DataQualityIndicator> getQualityIndicators() {
		return qualityIndicators;
	}

	/**
	 * Convenience method for returning data quality indicators as List in order to user p:dataList (primefaces)
	 * 
	 * @return List of data quality indicators
	 */
	public List<DataQualityIndicator> getQualityIndicatorsAsList() {
		return new ArrayList<DataQualityIndicator>(this.getQualityIndicators());
	}

	@Override
	public Set<IDataQualityIndicator> getDataQualityIndicators() {
		// we use type erasure because we know the DataQualityIndicator always implements IDataQualityIndicator
		return (Set<IDataQualityIndicator>) (Set) qualityIndicators;
	}

	protected void setQualityIndicators(Set<DataQualityIndicator> qualityIndicators) {
		this.qualityIndicators = qualityIndicators;
	}

	public void addQualityIndicator(DataQualityIndicator qualityIndicator) {
		if (!qualityIndicators.contains(qualityIndicator))
			qualityIndicators.add(qualityIndicator);
	}

	@Override
	public List<IGlobalReference> getReferencesToReviewers() {
		List<IGlobalReference> reviewers = new ArrayList<IGlobalReference>();
		for (IGlobalReference reviewer : referencesToReviewers)
			reviewers.add(reviewer);
		return reviewers;
	}

	protected void setReferencesToReviewers(Set<GlobalReference> referencesToReviewers) {
		this.referencesToReviewers = referencesToReviewers;
	}

	public void addReferenceToReviewers(GlobalReference referenceToReviewer) {
		if (!referencesToReviewers.contains(referenceToReviewer))
			referencesToReviewers.add(referenceToReviewer);
	}

	@Override
	public MultiLanguageText getReviewDetails() {
		return reviewDetails;
	}

	public void setReviewDetails(MultiLanguageText reviewDetails) {
		this.reviewDetails = reviewDetails;
	}

	@Override
	public Set<IScope> getScopes() {
		return scopes;
	}

	protected void setScopes(Set<IScope> scopes) {
		this.scopes = scopes;
	}

	public void addScope(ScopeOfReview scope) {
		if (!scopes.contains(scope))
			scopes.add(scope);
	}

	@Override
	public TypeOfReviewValue getType() {
		return type;
	}

	public void setType(TypeOfReviewValue type) {
		this.type = type;
	}

	@Override
	public int hashCode() {
		int hash = 0;
		hash += (id != null ? id.hashCode() : 0);
		return hash;
	}

	@Override
	public boolean equals(Object object) {
		// TODO: Warning - this method won't work in the case the id fields are not set
		if (!(object instanceof Review)) {
			return false;
		}
		Review other = (Review) object;
		if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.process.Review[id=" + id + "]";
	}

}
