/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package de.iai.ilcd.webgui.controller.admin;

import de.iai.ilcd.configuration.ConfigurationService;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.LockedAccountException;
import org.apache.shiro.authc.UnknownAccountException;

import de.iai.ilcd.model.dao.MergeException;
import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.security.UserLoginActions;
import org.apache.commons.configuration.Configuration;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean
@RequestScoped
public class ActivationHandler {

    private String userName = null;
    private String activationKey = null;
    private UserDao userDao = new UserDao();
    User user = null;

    public ActivationHandler() {
        // check if the page was called with a request parameter specifying the current user
        userName = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get("user");
        activationKey = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get("activationKey");
        FacesMessage message = new FacesMessage();

        if (userName == null) {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
                    "No valid registered user name for user activation.", "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        if (activationKey == null) {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
                    "The activation key is not valid for this user" + userName, "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        user = userDao.getUser(userName);
        if (user == null) {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
                    "There is no registrated user with name " + userName, "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        if (user.getRegistrationKey() == null) {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "The user account with user name "
                    + userName + " is already activated!", "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        if (!user.getRegistrationKey().equals(activationKey)) {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
                    "Invalid activation key! Please notify the site administrator.", "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        // clear registration key to activate user
        user.setRegistrationKey(null);
        try {
            userDao.merge(user);
        } catch (MergeException ex) {
            FacesMessage facesMsg = new FacesMessage(
                    FacesMessage.SEVERITY_WARN,
                    "Sorry, but the system cannot save the activated state to the database; please try again later.",
                    "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            return;
        }
        
        Configuration properties = ConfigurationService.getProperties();
        boolean selfActivation = properties.getBoolean("user.registration.selfActivation", Boolean.FALSE);

        String facesMessage = "";

        if (selfActivation) {
            UserLoginActions loginActions = new UserLoginActions();

            try {
                loginActions.login(user.getUserName(), user.getPassword());
            } catch (UnknownAccountException uae) {
                message.setSummary("A login with this login name doesn't exist");
                message.setSeverity(FacesMessage.SEVERITY_ERROR);
                return;
            } catch (IncorrectCredentialsException ice) {
                message.setSummary("The password given is wrong");
                message.setSeverity(FacesMessage.SEVERITY_ERROR);
                return;
            } catch (LockedAccountException lae) {
                message.setSummary("Your account is currently locked. Contact an administrator");
                message.setSeverity(FacesMessage.SEVERITY_ERROR);
                return;
            } catch (Exception e) {
                message.setSummary("An error occurred; Please try again");
                message.setSeverity(FacesMessage.SEVERITY_ERROR);
                message.setDetail(e.getMessage());
                return;
            }
            
            facesMessage = "Your account is now activated and you are already logged into the system";
        } else {
            facesMessage = "The account is now activated";
        }

        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, facesMessage, "");
        FacesContext.getCurrentInstance().addMessage("info", facesMsg);
    }

    public String getActivationKey() {
        return activationKey;
    }

    public String getUserName() {
        return userName;
    }
}
