/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.velocity.tools.generic.ValueParser;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.model.LazyDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.dao.DeleteDataSetException;
import de.iai.ilcd.model.dao.LCIAMethodDao;
import de.iai.ilcd.model.lciamethod.LCIAMethod;

/**
 * Handler for admin LCIAMethod List
 * TODO: Refactor bean structure for admin similar to public frontend. Lots of common functionality!
 */
@ManagedBean(name="lciaMethodListHandler")
@ViewScoped
public class LCIAMethodListHandler implements Serializable {

	/**
	 * Serialization ID
	 */
	private static final long serialVersionUID = -6771633832246107270L;

	/**
	 * Logger
	 */
	private static Logger logger = LoggerFactory.getLogger(LCIAMethodListHandler.class);

	/**
	 * Lazy model for UI
	 */
	private LazyDataModel<LCIAMethod> lazyModel = new LCIAMethodDataModel();

	/**
	 * Seleted LCIA Methods
	 */
	private LCIAMethod[] selectedLCIAMethods;

	/**
	 * Parameter parser
	 */
	private final ValueParser params = new ValueParser();

	/**
	 * Data access object instance
	 */
	private LCIAMethodDao dao = new LCIAMethodDao();
	
	/**
	 * Create the handler
	 */
	public LCIAMethodListHandler() {
		lazyModel.setRowCount((int) this.dao.searchResultCount(this.params) );
	}

	/**
	 * Get the lazy data model
	 * @return lazy data model
	 */
	public LazyDataModel<LCIAMethod> getLazyModel() {
		return lazyModel;
	}

	/**
	 * Set the lazy data model
	 * @param lazyModel lazy model to set
	 */
	public void setLazyModel(LazyDataModel<LCIAMethod> lazyModel) {
		this.lazyModel = lazyModel;
	}

	/**
	 * Get the selected LCIA methods
	 * @return selected LCIA methods
	 */
	public LCIAMethod[] getSelectedLCIAMethods() {
		return this.selectedLCIAMethods;
	}

	/**
	 * Set the selected LCIA methods
	 * @param selectedLCIAMethods
	 */
	public void setSelectedLCIAMethods(LCIAMethod[] selectedLCIAMethods) {
		this.selectedLCIAMethods = selectedLCIAMethods;
	}

	/**
	 * Delete the selected LCIA methods
	 */
	public void deleteSelected() {
		if (this.selectedLCIAMethods == null || this.selectedLCIAMethods.length == 0 ) {
			return;
		}

		for (LCIAMethod method : this.selectedLCIAMethods) {
			try {
				this.dao.remove(method);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Could not remove LCIA method "
						+ method.getName().getDefaultValue() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed LCIA method "
					+ method.getName().getDefaultValue() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}

		this.selectedLCIAMethods = null;

		this.lazyModel.setRowCount( (int) this.dao.searchResultCount(this.params) );
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:lciaMethodTable");
		if (dataTable != null){
			dataTable.loadLazyData();
		}
	}

	/**
	 * Lazy data model for the LCIA method listing
	 */
	private class LCIAMethodDataModel extends LazyDataModel<LCIAMethod> {
		
		/**
		 * Serialization ID
		 */
		private static final long serialVersionUID = 844251019282605363L;

		/**
		 * {@inheritDoc}
		 */
		@Override
		public List<LCIAMethod> load(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filters) {
			logger.info("Loading the lazy LCIA method data between {0} and {1}", new Object[] { first, (first + pageSize) });

			List<LCIAMethod> lazyLCIAMethods = new ArrayList<LCIAMethod>();

			LCIAMethodDao dao = new LCIAMethodDao();

			lazyLCIAMethods = dao.search(LCIAMethod.class, LCIAMethodListHandler.this.params, first, pageSize, sortField);

			this.setRowCount( (int)dao.searchResultCount(LCIAMethodListHandler.this.params) );
			return lazyLCIAMethods;
		}
	}
}
